#!/usr/bin/perl
#
# DistOnTree
# This is a script to make distance matrix based on the distance on
# a phylogenetic tree. Official web site of this script is
# http://www.fifthdimension.jp/products/distontree/ .
# To know script details, see above URL.
# 
# Copyright (C) 2006-2007  Akifumi S. Tanabe
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
# 
# Input file must be AltNEXUS format without node labels and contains
# only 1 tree.
# Output file is tab-deliminated text.


my $buildno = '1.0.2007.02.19';

print << "_END";
DistOnTree $buildno
=========================

This is a script to make distance matrix based on the distance on
a phylogenetic tree. Official web site of this script is
http://www.fifthdimension.jp/products/distontree/ .
To know script details, see above URL.

Copyright (C) 2006-2007  Akifumi S. Tanabe

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

_END

$treefile = shift(@ARGV);

my %Dist;
open (TREE, "< $treefile") or die "$filename: $!\n";
while (<TREE>) {
	if (s/\s*tree\s+(\S+)\s*=\s*(?:\[..\]\s*)?(\(.+);/$2/) {
		my $treename = $1;
		s/^\((.+)\)$/$1/;
		my %BrLensOTU;
		while (s/([^,:\(\)]+):(\d+\.?\d*)(e\D?\d+)?/$1/) {
			my $temp1 = $2;
			if ($3 =~ /e(\D)?0*(\d+)/) {
				$temp2 = $2;
				if ($1 eq '-') {
					$temp1 *= 10 ** ((-1) * $temp2);
				} else {
					$temp1 *= 10 ** $temp2;
				}
			}
			$BrLensOTU{$1} = $temp1;
		}
		my %BrLensCLADE;
		my $num = 0;
		while (s/\(([^:\(\)]+)\):(\d+\.?\d*)(e\D?\d+)?/<$num>$1<\/$num>/) {
			my $temp1 = $2;
			if ($3 =~ /e(\D)?0*(\d+)/) {
				$temp2 = $2;
				if ($1 eq '-') {
					$temp1 *= 10 ** ((-1) * $temp2);
				} else {
					$temp1 *= 10 ** $temp2;
				}
			}
			$BrLensCLADE{$num} = $temp1;
			$num++;
		}
		undef $num;
		while (s/(?:<\/?\d+>|,)*([^,<>\r\n]+),?//) {
			my $OTU1 = $1;
			my $tree = $_;
			my @Lens;
			while ($tree =~ s/([^,<>\r\n]+|<\/?\d+>),?//) {
				if ($1 =~ /<(\d+)>/) {
					push (@Lens, $BrLensCLADE{$1});
				} elsif ($1 =~ /<\/(\d+)>/) {
					if ($Lens[-1] eq $BrLensCLADE{$1}) {
						pop (@Lens);
					} else {
						push (@Lens, $BrLensCLADE{$1});
					}
				} elsif ($1 =~ /([^,<>\r\n]+)/) {
					my $sum;
					foreach (@Lens) {
						$sum += $_;
					}
					$Dist{"$OTU1,$1"} = ($BrLensOTU{$OTU1} + $sum + $BrLensOTU{$1});
				} else {
					print "ERROR!: Invalid tree file.\n";
					exit(0);
				}
			}
		}
	}
}
close (TREE);

open (OUT, "> $treefile" . ".dist") or die "$filename: $!\n";
print(OUT "from\tto\tdist\n");
foreach (keys(%Dist)) {
	my @OTU = split(/,/, $_);
	print(OUT "$OTU[0]\t$OTU[1]\t$Dist{$_}\n");
}
close (OUT);
